@@ -1,3 +1,5 @@
+from enum import Enum
+from http import HTTPStatus
 from unittest import mock
 from urllib.parse import urlencode
 
@@ -9,6 +11,10 @@ from django.test import override_settings
 from django_json_api.client import JSONAPIClient, JSONAPIClientError
 
 
+class NameValues(str, Enum):
+    value = "value"
+
+
 @pytest.fixture
 def mock_requests():
     with requests_mock.Mocker() as mocker:
@@ -138,6 +144,21 @@ def test_jsonapi_client_get_with_sort(mock_requests):
     assert mock_requests.last_request.url == url
 
 
+@pytest.mark.parametrize("value", ("value", NameValues.value))
+def test_jsonapi_client_get_with_filter(mock_requests, value):
+    expected_params = {
+        "fields[related_records]": "name,other_related",
+        "include": "other_related",
+        "filter[name]": "value",
+    }
+    url = f"http://example.com/related-records/?{urlencode(expected_params)}"
+    mock_requests.get(url, json="{}")
+    client = JSONAPIClient()
+    client.get("related_records", filters={"name": value})
+    assert mock_requests.called
+    assert mock_requests.last_request.url == url
+
+
 def test_jsonapi_client_get_with_page_size(mock_requests):
     expected_params = {
         "fields[related_records]": "name,other_related",
@@ -187,3 +208,37 @@ def test_jsonapi_get_unresolved_resource():
     client = JSONAPIClient()
     with pytest.raises(JSONAPIClientError, match=r'Cannot resolve resource "unresolvable"'):
         client.get("unresolvable")
+
+
+def test_jsonapi_client_patch(mock_requests):
+    mock_requests.register_uri(
+        "PATCH",
+        "http://test/api/tests/1/",
+        status_code=HTTPStatus.OK,
+        json={
+            "data": {
+                "type": "tests",
+                "id": "1",
+                "attribute": {
+                    "field": "value",
+                    "other_field": "other_value",
+                },
+            }
+        },
+    )
+    client = JSONAPIClient()
+    client.patch(resource_type="tests", resource_id=1, attributes={"field": "value"})
+    assert mock_requests.called
+
+
+def test_jsonapi_client_patch__error_response(mock_requests):
+    mock_requests.register_uri(
+        "PATCH",
+        "http://test/api/tests/1/",
+        status_code=HTTPStatus.NOT_FOUND,
+        json="Record not found",
+    )
+    client = JSONAPIClient()
+    with pytest.raises(JSONAPIClientError):
+        client.patch(resource_type="tests", resource_id=1, attributes={"field": "value"})
+    assert mock_requests.called
